"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const api_convention_binder_1 = require("./api-convention-binder");
const decorator_binder_1 = require("./decorator-binder");
const default_binder_1 = require("./default-binder");
const val_type_binder_1 = require("./val-type-binder");
class ParameterBinder {
    constructor(routeInfo, pathResolver) {
        this.routeInfo = routeInfo;
        this.pathResolver = pathResolver;
        this.commands = [];
        //priorities
        this.commands = [
            new decorator_binder_1.DecoratorBinder(),
            new api_convention_binder_1.ApiConventionBinder(),
            new val_type_binder_1.ValTypeBinder(this.pathResolver),
            new default_binder_1.DefaultBinder()
        ];
    }
    getParameters(context) {
        if (!this.routeInfo.methodMetaData.parameters ||
            this.routeInfo.methodMetaData.parameters.length == 0)
            return [];
        let result = [];
        for (let par of this.routeInfo.methodMetaData.parameters) {
            result.push(this.bind(par.name, context));
        }
        return result;
    }
    bind(parameterName, request) {
        for (let cmd of this.commands) {
            let result = cmd.bind(this.routeInfo, parameterName, request);
            if (result.type == "Exit")
                return result.value;
        }
    }
}
exports.ParameterBinder = ParameterBinder;
