"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const base_converter_1 = require("./base-converter");
const router_1 = require("../../router");
const kamboja_core_1 = require("kamboja-core");
class DecoratedValueConverter extends base_converter_1.BaseConverter {
    convert(meta, value) {
        let decorators = meta.decorators && meta.decorators.filter(x => this.filterTypes(x));
        if (!decorators || decorators.length == 0)
            return this.next();
        if (typeof value == "undefined" || value == null)
            return this.exit(value);
        let type = this.getParameter(decorators[0]);
        if (typeof value == "object" && !Array.isArray(value))
            throw new Error(`Expected parameter type of [@type('${type}') ${meta.name}] but got object in ${router_1.getRouteDetail(this.routeInfo)}`);
        switch (type) {
            case "string":
                return this.exit(value.toString());
            case "number":
                return this.exit(parseInt(value));
            case "boolean":
                return this.exit(value.toLowerCase() === "true");
        }
        //all array types (string[], number[], boolean[]) is passed to the default conversion
        return this.next();
    }
    filterTypes(x) {
        return (x.name == "type" &&
            kamboja_core_1.ValidationTypesAccepted.some(y => this.getParameter(x) == y));
    }
    getParameter(x) {
        return x.parameters[0].value
            .toString()
            .toLowerCase();
    }
}
exports.DecoratedValueConverter = DecoratedValueConverter;
