"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const Core = require("kamboja-core");
const validator_1 = require("../../validator");
const factory_1 = require("../factory");
function createController(option, controllerInfo, parameters) {
    let validator = new validator_1.ValidatorImpl(option.metaDataStorage, option.validators);
    validator.setValue(parameters, controllerInfo.classMetaData, controllerInfo.methodMetaData.name);
    let controller = factory_1.ControllerFactory.resolve(controllerInfo, option.dependencyResolver);
    controller.validator = validator;
    return controller;
}
class ControllerInvocation extends Core.Invocation {
    proceed() {
        let controller = createController(this.facade, this.controllerInfo, this.parameters);
        controller.context = this.context;
        let method = controller[this.controllerInfo.methodMetaData.name];
        let result;
        if (this.facade.autoValidation && !controller.validator.isValid())
            result = new Core.ActionResult(controller.validator.getValidationErrors(), 400, "application/json");
        else
            result = method.apply(controller, this.parameters);
        return this.createResult(result);
    }
    createResult(result) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let awaitedResult = yield Promise.resolve(result);
            if (awaitedResult instanceof Core.ActionResult)
                return awaitedResult;
            if (this.controllerInfo.classMetaData.baseClass == "ApiController") {
                return new Core.ActionResult(awaitedResult, 200, "application/json");
            }
            return new Core.ActionResult(awaitedResult, 200, "text/html");
        });
    }
}
exports.ControllerInvocation = ControllerInvocation;
