"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const Core = require("kamboja-core");
const invocation_1 = require("./invocation");
const factory_1 = require("./factory");
const framework_1 = require("../framework");
const index_1 = require("../binder/index");
class RequestHandler {
    constructor(facade, controllerInfo) {
        this.facade = facade;
        this.controllerInfo = controllerInfo;
        let rawGlobalMiddleware = (facade.middlewares || []).slice().reverse();
        let middlewares = factory_1.MiddlewareFactory.resolve(rawGlobalMiddleware, facade.dependencyResolver);
        if (controllerInfo) {
            let controller = factory_1.ControllerFactory.resolve(controllerInfo, facade.dependencyResolver);
            middlewares.push(...factory_1.MiddlewareFactory.resolve(framework_1.MiddlewareDecorator
                .getMiddlewares(controller), facade.dependencyResolver));
            middlewares.push(...factory_1.MiddlewareFactory.resolve(framework_1.MiddlewareDecorator
                .getMiddlewares(controller, controllerInfo.methodMetaData.name), facade.dependencyResolver));
        }
        this.tail = this.head = new invocation_1.AttachableInvocation(middlewares, controllerInfo, facade);
        middlewares.forEach(x => {
            let invocation = new invocation_1.MiddlewareInvocation(x, this.tail);
            invocation.middlewares = middlewares;
            invocation.controllerInfo = controllerInfo;
            invocation.facade = facade;
            this.tail = invocation;
        });
    }
    execute(context, response, invocation) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                this.head.attach(invocation);
                if (context.contextType == "HttpRequest" && this.controllerInfo) {
                    let binder = new index_1.ParameterBinder(this.controllerInfo, this.facade.pathResolver);
                    this.tail.parameters = binder.getParameters(context);
                }
                else if (context.contextType == "Handshake") {
                    this.tail.parameters = [context.getPacket()];
                }
                this.tail.context = context;
                let result = yield this.tail.proceed();
                if (context.contextType == "Handshake" && result.engine != "General")
                    throw new Error(`ActionResult Error, only return value of type 'redirect' and 'emit' are allowed in real time action`);
                yield result.execute(context, response, invocation.controllerInfo);
            }
            catch (e) {
                response.send(new Core.ActionResult(e.message, e.status || 500));
            }
        });
    }
}
exports.RequestHandler = RequestHandler;
