"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const missing_action_parameter_1 = require("./missing-action-parameter");
const missing_route_parameter_1 = require("./missing-route-parameter");
const non_associated_parameter_1 = require("./non-associated-parameter");
const duplicate_route_1 = require("./duplicate-route");
const conflict_internal_decorator_1 = require("./conflict-internal-decorator");
const convention_fail_1 = require("./convention-fail");
const class_not_controller_1 = require("./class-not-controller");
const class_not_exported_1 = require("./class-not-exported");
const reserved_word_used_1 = require("./reserved-word-used");
const event_with_query_parameters_1 = require("./event-with-query-parameters");
class RouteAnalyzer {
    constructor(routes) {
        this.routes = routes;
        this.commands = [
            new missing_action_parameter_1.MissingActionParameterAnalyzer(),
            new missing_route_parameter_1.MissingRouteParameterAnalyzer(),
            new non_associated_parameter_1.UnassociatedParameterAnalyzer(),
            new duplicate_route_1.DuplicateRouteAnalyzer(),
            new conflict_internal_decorator_1.ConflictInternalDecoratorAnalyzer(),
            new convention_fail_1.ConventionFailDecoratorAnalyzer(),
            new class_not_controller_1.ClassNotInherritedFromControllerAnalyzer(),
            new class_not_exported_1.ClassNotExportedControllerAnalyzer(),
            new reserved_word_used_1.ReservedWordUsedAnalyzer(),
            new event_with_query_parameters_1.EventWithQueryParameters()
        ];
    }
    analyse() {
        let result = [];
        for (let route of this.routes)
            for (let command of this.commands) {
                let analysis = command.analyse(route);
                if (analysis)
                    result = result.concat(analysis);
            }
        return result;
    }
}
exports.RouteAnalyzer = RouteAnalyzer;
