"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const qualified_name_1 = require("../resolver/qualified-name");
const Fs = require("fs");
const Path = require("path");
const Kecubung = require("kecubung");
const Babylon = require("babylon");
function flatten(metaList, fileName) {
    let result = [];
    metaList.forEach(x => {
        switch (x.type) {
            case "Module":
                let file = x;
                let clazz = flatten(file.children, fileName);
                if (clazz && clazz.length > 0) {
                    clazz.forEach(cls => {
                        cls.qualifiedClassName = file.name + "." + cls.qualifiedClassName;
                    });
                    result = result.concat(clazz);
                }
                break;
            case "Class":
                let curClass = x;
                curClass.qualifiedClassName = `${curClass.name}, ${fileName}`;
                result.push(curClass);
                break;
        }
    });
    return result;
}
class MetaDataLoader {
    constructor(idResolver, pathResolver) {
        this.idResolver = idResolver;
        this.pathResolver = pathResolver;
        this.flatStorage = {};
        this.storage = {};
    }
    load(path, category) {
        let files;
        if (typeof path == "string")
            files = this.getFilePaths([path], category);
        else
            files = this.getFilePaths(path, category);
        let flatResult = [];
        let result = [];
        for (let file of files) {
            let code = Fs.readFileSync(file).toString();
            let ast = Babylon.parse(code);
            let fileName = this.pathResolver.normalize(file);
            let meta = Kecubung.transform("ASTree", ast, fileName);
            flatResult = flatResult.concat(flatten(meta.children, meta.name));
            result.push(meta);
        }
        this.flatStorage[category] = flatResult;
        this.storage[category] = result;
    }
    getFilePaths(paths, category) {
        let result = [];
        for (const path of paths) {
            const fileDirectory = this.pathResolver.resolve(path);
            if (!Fs.existsSync(fileDirectory) && category == "Controller")
                throw Error(`Directory not found [${path}]`);
            if (!Fs.existsSync(fileDirectory) && category == "Model")
                continue;
            const resultPaths = Fs.readdirSync(fileDirectory)
                .filter(x => Path.extname(x) == ".js")
                .map(x => Path.join(fileDirectory, x));
            result = result.concat(resultPaths);
        }
        return result;
    }
    getFiles(category) {
        return this.storage[category];
    }
    getClasses(category) {
        return this.flatStorage[category];
    }
    get(classId) {
        let request = new qualified_name_1.QualifiedName(classId, this.pathResolver);
        for (let key in this.flatStorage) {
            let result = this.flatStorage[key].filter(x => {
                let qualified = new qualified_name_1.QualifiedName(x.qualifiedClassName, this.pathResolver);
                return request.equals(qualified);
            });
            if (result.length > 0)
                return result[0];
        }
    }
}
exports.MetaDataLoader = MetaDataLoader;
