"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const Core = require("kamboja-core");
const transformer_base_1 = require("./transformer-base");
class ApiConventionTransformer extends transformer_base_1.TransformerBase {
    constructor() {
        super(...arguments);
        this.conventions = ["get", "list", "add", "replace", "modify", "delete"];
    }
    transform(meta, parent, prevResult) {
        if (prevResult) {
            return this.next(prevResult);
        }
        if (this.conventions.some(x => x == meta.name)) {
            //if it doesn't contain parameter pass to other transformer
            if (meta.parameters.length < 1) {
                return this.next({
                    analysis: [Core.RouteAnalysisCode.ConventionFail],
                    initiator: "ApiConvention",
                    methodMetaData: meta,
                    overrideRequest: Core.OverrideRequest.Route,
                    //GET by design
                    httpMethod: "GET"
                });
            }
            switch (meta.name) {
                case "list":
                    return this.noParam(meta, parent, "GET");
                case "get":
                    return this.singleParam(meta, parent, "GET");
                case "delete":
                    return this.singleParam(meta, parent, "DELETE");
                case "replace":
                    return this.singleParam(meta, parent, "PUT");
                case "modify":
                    return this.singleParam(meta, parent, "PATCH");
                case "add":
                    return this.noParam(meta, parent, "POST");
            }
        }
        return this.next();
    }
    singleParam(meta, parent, method) {
        let path = "/:" + meta.parameters[0].name;
        //auto assigned required validation on id
        if (!meta.parameters[0].decorators)
            meta.parameters[0].decorators = [];
        if (!meta.parameters[0].decorators.some(x => x.name == "required")) {
            meta.parameters[0].decorators.push({
                type: 'Decorator',
                name: 'required',
                analysis: 1,
                location: { start: 1124, end: 1164 },
                parameters: []
            });
        }
        return this.exit({
            httpMethod: method,
            initiator: "ApiConvention",
            methodMetaData: meta,
            route: parent + path,
            methodPath: path
        });
    }
    noParam(meta, parent, method) {
        return this.exit({
            httpMethod: method,
            initiator: "ApiConvention",
            methodMetaData: meta,
            route: parent,
            methodPath: "/"
        });
    }
}
tslib_1.__decorate([
    transformer_base_1.when("Method")
], ApiConventionTransformer.prototype, "transform", null);
exports.ApiConventionTransformer = ApiConventionTransformer;
