"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const Core = require("kamboja-core");
const transformer_base_1 = require("./transformer-base");
class EventDecoratorTransformer extends transformer_base_1.TransformerBase {
    transform(meta, parent, prevResult) {
        if (prevResult) {
            //too complex to handle,
            //just pass previous result
            return this.next(prevResult);
        }
        if (meta.decorators && meta.decorators.length > 0 && meta.decorators.some(x => x.name == "on")) {
            let decorators = meta.decorators.filter(x => x.name == "on");
            let infos = decorators.map(decorator => {
                return this.createInfo(meta, decorator);
            });
            return this.exit(infos);
        }
        else
            return this.next();
    }
    createInfo(meta, decorator) {
        let analysis = [];
        let method = "EVENT";
        let route = decorator.parameters[0].value;
        let routeAnalysis = this.checkIfQueryParameterNotAllowed(route, method);
        if (routeAnalysis)
            analysis.push(routeAnalysis);
        let info = {
            initiator: "HttpMethodDecorator",
            httpMethod: method,
            methodMetaData: meta,
            route: route
        };
        if (analysis.length > 0)
            info.analysis = analysis;
        return info;
    }
    checkIfQueryParameterNotAllowed(route, method) {
        let routeParameters = route.split("/").filter(x => x.charAt(0) == ":");
        if (routeParameters.length > 0)
            return Core.RouteAnalysisCode.QueryParameterNotAllowed;
    }
}
tslib_1.__decorate([
    transformer_base_1.when("Method")
], EventDecoratorTransformer.prototype, "transform", null);
exports.EventDecoratorTransformer = EventDecoratorTransformer;
