"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const META_DATA_KEY = "kamboja:Call.when";
function when(kind) {
    return function (target, method, descriptor) {
        Reflect.defineMetadata(META_DATA_KEY, kind, target, method);
    };
}
exports.when = when;
function getWhen(target, methodName) {
    return Reflect.getMetadata(META_DATA_KEY, target, methodName);
}
exports.getWhen = getWhen;
class TransformerBase {
    transformChildren(children, parent) {
        let result = [];
        let lastResult;
        for (let child of children) {
            let trans = this.transformers.filter(x => getWhen(x, "transform") == child.type);
            for (let transformer of trans) {
                let tempResult = transformer.transform(child, parent, lastResult ? lastResult.info : undefined);
                let exit = false;
                switch (tempResult.status) {
                    case "ExitWithResult":
                        result = result.concat(tempResult.info);
                        exit = true;
                        break;
                    case "Next":
                        lastResult = tempResult;
                        break;
                    case "Exit":
                        exit = true;
                        break;
                }
                if (exit)
                    break;
            }
        }
        return result;
    }
    next(result) {
        if (result)
            return {
                status: "Next",
                info: Array.isArray(result) ? result : [result]
            };
        else
            return { status: "Next" };
    }
    exit(result) {
        if (result)
            return {
                status: "ExitWithResult",
                info: Array.isArray(result) ? result : [result]
            };
        return { status: "Exit" };
    }
}
exports.TransformerBase = TransformerBase;
