"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("reflect-metadata");
const MetaDataKey = "kamboja:Validator";
function decoratorName(decoratorName) {
    return (target, propertyKey, index) => {
        Reflect.defineMetadata(MetaDataKey, decoratorName, target);
    };
}
exports.decoratorName = decoratorName;
function getDecoratorName(target) {
    return Reflect.getMetadata(MetaDataKey, target);
}
exports.getDecoratorName = getDecoratorName;
class ValidatorBase {
    validate(args) {
        return undefined;
    }
    isEmpty(value) {
        return typeof value == "undefined"
            || value == null
            || (typeof value == "string" && value.trim() == "");
    }
    validateFields(arg) {
        if (arg.type == "PropertiesValidator") {
            let result = [];
            for (let property of arg.classInfo.properties) {
                if (arg.isArray) {
                    if (!Array.isArray(arg.classInstance))
                        return [{
                                field: arg.parentField,
                                message: `[${arg.parentField}] must be a type of Array`
                            }];
                    let instances = arg.classInstance;
                    for (let i = 0; i < instances.length; i++) {
                        let instance = instances[i];
                        let value = instance[property.name];
                        let valResult = this.useValidators({
                            decorators: property.decorators,
                            value: value,
                            field: property.name,
                            parentField: `${arg.parentField}[${i}]`,
                            classInfo: arg.classInfo
                        });
                        result = result.concat(valResult);
                    }
                }
                else {
                    let value = arg.classInstance[property.name];
                    let valResult = this.useValidators({
                        decorators: property.decorators,
                        value: value,
                        field: property.name,
                        parentField: arg.parentField,
                        classInfo: arg.classInfo
                    });
                    result = result.concat(valResult);
                }
            }
            return result.length == 0 ? undefined : result;
        }
        else {
            let result = [];
            let method = arg.classInfo.methods.filter(x => x.name == arg.methodName)[0];
            for (let i = 0; i < method.parameters.length; i++) {
                let parameterMetadata = method.parameters[i];
                let value = arg.parameterValues[i];
                let valResult = this.useValidators({
                    decorators: parameterMetadata.decorators,
                    classInfo: arg.classInfo,
                    field: parameterMetadata.name,
                    parentField: arg.parentField,
                    value: value
                });
                if (valResult)
                    result = result.concat(valResult);
            }
            return result.length == 0 ? undefined : result;
        }
    }
    useValidators(arg) {
        let result = [];
        if (!arg.decorators)
            return;
        for (let decorator of arg.decorators) {
            let validators = this.validators.filter(x => getDecoratorName(x) == decorator.name);
            for (let validator of validators) {
                let validationResult = validator.validate({
                    classInfo: arg.classInfo,
                    decoratorArgs: decorator.parameters,
                    field: arg.field,
                    parentField: arg.parentField ? `${arg.parentField}` : arg.parentField,
                    value: arg.value
                });
                if (validationResult)
                    result = result.concat(validationResult);
            }
        }
        return result;
    }
}
exports.ValidatorBase = ValidatorBase;
