"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const baseclasses_1 = require("./baseclasses");
class RangeValidator extends baseclasses_1.ValidatorBase {
    validate(args) {
        if (this.isEmpty(args.value))
            return;
        let fieldName = args.parentField ? `${args.parentField}.${args.field}` : args.field;
        let minValue = undefined, maxValue = undefined, customMessage = undefined;
        minValue = args.decoratorArgs[0].value;
        if (args.decoratorArgs[1])
            maxValue = args.decoratorArgs[1].value;
        if (args.decoratorArgs[2])
            customMessage = args.decoratorArgs[2].value;
        if (typeof args.value == "number") {
            let numericValue = args.value;
            if (numericValue < minValue) {
                return [{
                        field: fieldName,
                        message: customMessage || `[${args.field}] must be greater than ${minValue}`
                    }];
            }
            if (maxValue && numericValue > maxValue) {
                return [{
                        field: fieldName,
                        message: customMessage || `[${args.field}] must be less than ${maxValue}`
                    }];
            }
        }
        else if (typeof args.value == "string") {
            let length = args.value.length;
            if (length < minValue) {
                return [{
                        field: fieldName,
                        message: customMessage || `[${args.field}] length must be more than ${minValue} characters`
                    }];
            }
            if (maxValue && length > maxValue) {
                return [{
                        field: fieldName,
                        message: customMessage || `[${args.field}] length must be less than ${maxValue} characters`
                    }];
            }
        }
    }
}
tslib_1.__decorate([
    baseclasses_1.decoratorName("range")
], RangeValidator.prototype, "validate", null);
exports.RangeValidator = RangeValidator;
