"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const baseclasses_1 = require("./baseclasses");
const kamboja_core_1 = require("kamboja-core");
const qualified_name_1 = require("../resolver/qualified-name");
class TypeValidator extends baseclasses_1.ValidatorBase {
    constructor(storage) {
        super();
        this.storage = storage;
    }
    validate(args) {
        const FIELD_NAME = args.parentField ? `${args.parentField}.${args.field}` : args.field;
        if (this.isEmpty(args.value))
            return;
        if (!args.decoratorArgs)
            return;
        let decoratorArg = args.decoratorArgs[0];
        if (this.isEmpty(decoratorArg.value))
            throw new Error(`Qualified class name should be specified in @type in [${args.classInfo.name}]`);
        if (kamboja_core_1.ValidationTypesAccepted.some(x => x == decoratorArg.value.toLowerCase()))
            return;
        let qualified = new qualified_name_1.QualifiedName(decoratorArg.value, this.storage.pathResolver);
        if (!qualified.isValid())
            throw new Error(`Invalid qualified class name [${decoratorArg.value}] in @type decorator in [${args.classInfo.name}]`);
        let clazz = this.storage.get(qualified.qualifiedName);
        if (!clazz)
            throw new Error(`Class [${decoratorArg.value}] in @type is no found in [${args.classInfo.name}]`);
        return this.validateFields({
            type: "PropertiesValidator",
            classInfo: clazz,
            parentField: args.parentField ? `${args.parentField}.${args.field}` : args.field,
            classInstance: args.value,
            isArray: qualified.isArray()
        });
    }
}
tslib_1.__decorate([
    baseclasses_1.decoratorName("type")
], TypeValidator.prototype, "validate", null);
exports.TypeValidator = TypeValidator;
