"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const baseclasses_1 = require("./baseclasses");
const type_validator_1 = require("./type-validator");
const required_validator_1 = require("./required-validator");
const email_validator_1 = require("./email-validator");
const range_validator_1 = require("./range-validator");
class ValidatorImpl extends baseclasses_1.ValidatorBase {
    constructor(metaDataStorage, validators) {
        super();
        this.metaDataStorage = metaDataStorage;
        this.errors = [];
        if (!validators)
            validators = [];
        this.validators = validators.slice();
        let typeValidator = new type_validator_1.TypeValidator(metaDataStorage);
        typeValidator.validators = this.validators;
        this.validators.push(typeValidator);
        this.validators.push(new email_validator_1.EmailValidator());
        this.validators.push(new range_validator_1.RangeValidator());
        this.validators.push(new required_validator_1.RequiredValidator());
    }
    setValue(parameters, meta, methodName) {
        this.parameters = parameters;
        this.classMetaData = meta;
        this.methodName = methodName;
    }
    isValid() {
        let result = this.validateFields({
            type: "ParametersValidator",
            classInfo: this.classMetaData,
            methodName: this.methodName,
            parameterValues: this.parameters,
        });
        if (result)
            this.errors = result;
        return this.errors.length == 0;
    }
    getValidationErrors() {
        return this.errors.length > 0 ? this.errors : undefined;
    }
}
exports.ValidatorImpl = ValidatorImpl;
